developmentIntegration = new DevelopmentIntegrationTool()

/**
 * Velocity tool to get development-related information from the Jira Development Integration system app.
 */
class DevelopmentIntegrationTool {
	/**
	 * Returns the development information as a one-line summary.
	 */
	def static getSummary(def customFieldValue) {
		def summary = customFieldValue?.summaryBean?.summary
		return summary ? developmentSummaryToString(summary) : null
	}

	private static def developmentSummaryToString(def summary) {
		// see https://confluence.atlassian.com/jirasoftwareserver/viewing-the-development-information-for-an-issue-938845580.html
		def entityNames = [
				"branch" : ["branch", "branches"],
				"repository" : ["commit", "commits"], // misleading property name
				"pullrequest" : ["pull request", "pull requests"]
		]
		def result = []

		// branches
		def branchCount = summary.branch.overall.count
		if (branchCount) {
			result << (branchCount + " " + entityNames["branch"][(branchCount == 1) ? 0 : 1])
		}

		// commits
		def commitCount = summary.repository.overall.count
		if (commitCount) {
			result << (commitCount + " " + entityNames["repository"][(commitCount == 1) ? 0 : 1])
		}

		// pull requests
		def pullRequestOverAll = summary.pullrequest.overall
		def stateCount = pullRequestOverAll.stateCount
		if (stateCount) {
			if (pullRequestOverAll.hasProperty("details")) {
				def pullRequestStrings = []

				def openCount = pullRequestOverAll.details.openCount
				if (openCount) {
					pullRequestStrings << (openCount + " open")
				}
				def mergedCount = pullRequestOverAll.details.mergedCount
				if (mergedCount) {
					pullRequestStrings << (mergedCount + " merged")
				}
				def declinedCount = pullRequestOverAll.details.declinedCount
				if (declinedCount) {
					pullRequestStrings << (declinedCount + " declined")
				}

				def sumCount = openCount + mergedCount + declinedCount
				if (pullRequestStrings) {
					result << (pullRequestStrings.join(" + ") + " " + entityNames["pullrequest"][(sumCount == 1) ? 0 : 1])
				}
			} else {
				// details are not available in earlier Jira 7.x versions
				result << (stateCount + " " + entityNames["pullrequest"][(stateCount == 1) ? 0 : 1])
			}
		}

		return result.join(", ")
	}
}
