import java.awt.Color
import com.google.common.collect.HashMultiset
import com.google.common.collect.TreeMultiset
import org.jfree.chart.plot.PlotOrientation
import org.jfree.chart.renderer.xy.XYAreaRenderer
import org.jfree.data.category.CategoryDataset
import org.jfree.data.category.DefaultCategoryDataset
import org.jfree.data.general.PieDataset
import org.jfree.data.general.DefaultPieDataset
import org.jfree.data.time.Day
import org.jfree.data.time.TimeSeries
import org.jfree.data.time.TimeSeriesCollection


helloCharts = new HelloChartsTool()

/**
 * Demo tool for the chart rendering tutorial.
 * @see https://www.midori-global.com/products/better-pdf-exporter-for-jira/tutorials/charts
 */
public class HelloChartsTool {
	def static calendar = Calendar.instance

	def static CREATED_COLOR = new Color(0xde1721)
	def static RESOLVED_COLOR = new Color(0x45b63f)

	TimeSeriesCollection getCreatedVsResolvedDataset(issues) {
		def createdByDay = new TreeMultiset()
		def resolvedByDay = new TreeMultiset()
		issues.each {
			createdByDay.add(new Day(it.getCreated()))
			if(it.resolutionDate) {
				resolvedByDay.add(new Day(it.resolutionDate))
			}
		}

		def createdSeries = new TimeSeries("created")
		int cummulativeTotal = 0
		for(day in createdByDay.elementSet()) {
			cummulativeTotal += createdByDay.count(day)
			createdSeries.add(day, cummulativeTotal)
		}

		def resolvedSeries = new TimeSeries("resolved")
		cummulativeTotal = 0
		for(day in resolvedByDay.elementSet()) {
			cummulativeTotal += resolvedByDay.count(day)
			resolvedSeries.add(day, cummulativeTotal)
		}

		def dataset = new TimeSeriesCollection()
		dataset.addSeries(createdSeries)
		dataset.addSeries(resolvedSeries)

		return dataset
	}

	def getCreatedVsResolvedCustomizer() {
		return { chart ->
			chart.plot.renderer = new XYAreaRenderer()
			chart.plot.renderer.plotLines = true
			chart.plot.renderer.plotShapes = true
			chart.plot.renderer.plotArea = false

			chart.plot.renderer.setSeriesStroke(0, RenderingUtils.LINE_STROKE)
			chart.plot.renderer.setSeriesPaint(0, CREATED_COLOR)
			chart.plot.renderer.setSeriesShape(0, RenderingUtils.CIRCLE_SHAPE)

			chart.plot.renderer.setSeriesStroke(1, RenderingUtils.LINE_STROKE)
			chart.plot.renderer.setSeriesPaint(1, RESOLVED_COLOR)
			chart.plot.renderer.setSeriesShape(1, RenderingUtils.CIRCLE_SHAPE)
		}
	}

	PieDataset getIssueCountByStatusDataset(issues) {
		def statusCounts = new HashMultiset()
		issues.each {
			statusCounts.add(it.statusObject.name)
		}

		def dataset = new DefaultPieDataset()
		statusCounts.elementSet().each {
			dataset.setValue(it, statusCounts.count(it))
		}

		return dataset
	}

	def getIssueCountByStatusCustomizer() {
		return { chart ->
			chart.plot.baseSectionOutlinePaint = RenderingUtils.BACKGROUND_COLOR
			chart.plot.drawingSupplier = new GradientDrawingSupplier(chart.plot.dataset.itemCount)
		}
	}

	CategoryDataset getIssuesByAssigneeDataset(issues) {
		def issuesByStatus = new TreeMultiset()
		issues.each {
			if(it.assignee) {
				issuesByStatus.add(it.assignee.displayName)
			} else {
				issuesByStatus.add("Unassigned")
			}
		}

		def dataset = new DefaultCategoryDataset()
		issuesByStatus.elementSet().each {
			dataset.setValue(issuesByStatus.count(it), "count", it)
		}

		return dataset
	}

	def getIssuesByAssigneeCustomizer() {
		return { chart ->
			chart.plot.orientation = PlotOrientation.HORIZONTAL
			chart.plot.renderer.setSeriesPaint(0, RESOLVED_COLOR)
			chart.plot.renderer.setSeriesOutlinePaint(0, RenderingUtils.BACKGROUND_COLOR)
		}
	}
}

