import com.atlassian.jira.component.ComponentAccessor

issueHelper = new IssueHelperTool()

/**
 * Velocity tool that provides general helper methods for working with issues.
 */
class IssueHelperTool {
	def customFieldManager = ComponentAccessor.customFieldManager
	def issueManager = ComponentAccessor.issueManager

	def getIssueByKey(def issueKey) {
		issueManager.getIssueObject(issueKey)
	}

	def getIssueSummaryByKey(def issueKey) {
		getIssueByKey(issueKey)?.summary ?: ''
	}

	/**
	 * Returns a mapping of custom field type keys to custom fields for the passed issue.
	 */
	def getCustomFieldsByTypes(def issue) {
		def customFields = new HashMap()
		for (def customField in customFieldManager.getCustomFieldObjects(issue)) {
			customFields[customField.customFieldType.key] = customField
		}
		return customFields
	}

	/**
	 * Calls a method of an object with given arguments.
	 * Useful when the method is inaccessible in the Velocity templates.
	 */
	def invoke(def object, def method, def args) {
		object.invokeMethod(method, args as Object[])
	}

	/**
	 * Converts a list of key-value pairs to a LinkedHashMap (preserves insertion-order).
	 */
	def toLinkedHashMap(String... args) {
		def map = new LinkedHashMap()
		for (def i = 0; i < args.length; i += 2) {
			map[args[i]] = args[i + 1]
		}
		return map
	}
}