import com.atlassian.jira.component.ComponentAccessor
import org.apache.log4j.Logger

scriptRunner = new ScriptRunnerTool()

/**
 * ScriptRunnerTool tool to get data from ScriptRunner custom fields.
 */
class ScriptRunnerTool {
	def log = Logger.getLogger(this.getClass())

	def scriptFieldConfigurations
	def fieldIdToModelTemplate
	def fieldIdToSearcherKey = [:]
	def scriptFieldIdsWithCustomTemplate

	/**
	 * Returns the model template for the given custom field.
	 */
	def getModelTemplate(customField) {
		try {
			if (fieldIdToModelTemplate == null) {
				fieldIdToModelTemplate = [:]
				getScriptFieldConfigurations().each { k, v ->
					fieldIdToModelTemplate[v.hasProperty("customFieldId") ? v.customFieldId : v.id] = v.modelTemplate
				}
			}
			return fieldIdToModelTemplate[customField.idAsLong]
		} catch (Exception ex) {
			log.warn("Failed to get the modelTemplate for ${customField.id}", ex)
			return ""
		}
	}

	/**
	 * Returns the searcher key for the script field or a blank string if it uses a custom template.
	 */
	def getSearcherKeyOrBlank(customField) {
		try {
			return !isScriptFieldWithCustomTemplate(customField) ? getSearcherKey(customField) : ""
		} catch (Exception ex) {
			log.warn("Failed to get the searcherKey or modelTemplate for ${customField.id}", ex)
			return ""
		}
	}

	/**
	 * Returns true if the custom field uses a custom template.
	 */
	private def isScriptFieldWithCustomTemplate(customField) {
		if (scriptFieldIdsWithCustomTemplate == null) {
			// find
			scriptFieldIdsWithCustomTemplate = getScriptFieldConfigurations()
					.findAll { k, v -> (v.modelTemplate == "custom") && (!v.metaClass.respondsTo(v, "isCannedScript") || !v.isCannedScript()) } // second condition is required in modern ScriptRunner versions
					.collect { k, v -> v.hasProperty("customFieldId") ? v.customFieldId : v.id } // property has been renamed
		}

		return scriptFieldIdsWithCustomTemplate.contains(customField.idAsLong)
	}

	private def getSearcherKey(customField) {
		def customFieldId = customField.idAsLong
		if (!fieldIdToSearcherKey.containsKey(customFieldId)) {
			fieldIdToSearcherKey[customFieldId] = customField.searcherRef.get().descriptor.completeKey
		}
		return fieldIdToSearcherKey[customFieldId]
	}

	private def getScriptFieldConfigurations() {
		if (scriptFieldConfigurations == null) {
			scriptFieldConfigurations = ComponentAccessor.getOSGiComponentInstanceOfType(ComponentAccessor.pluginAccessor.classLoader.findClass("com.onresolve.scriptrunner.runner.ScriptFieldManager")).configs
		}
		return scriptFieldConfigurations
	}
}
